import { __awaiter } from 'tslib';
import { createInstance, LOCALSTORAGE } from 'localforage';
import { SubscriptionClient } from 'subscriptions-transport-ws';
import { ApolloClient } from 'apollo-client';
import { createHttpLink } from 'apollo-link-http';
import { WebSocketLink } from 'apollo-link-ws';
import { InMemoryCache } from 'apollo-cache-inmemory';
import { split } from 'apollo-link';
import { getMainDefinition } from 'apollo-utilities';
import { delay } from 'bluebird';
import { Injectable, Component, NgModule, APP_INITIALIZER, Optional, SkipSelf } from '@angular/core';
import { Router, RouterModule } from '@angular/router';
import { ToastController, LoadingController, IonicModule } from '@ionic/angular';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * Generated from: lib/services/ec.service.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @record
 */
function ECLite() { }
if (false) {
    /** @type {?} */
    ECLite.prototype.fetch;
    /** @type {?} */
    ECLite.prototype.WebSocket;
}
class EdgeCentralConfig {
}
if (false) {
    /** @type {?} */
    EdgeCentralConfig.prototype.cloudCentralHost;
    /** @type {?} */
    EdgeCentralConfig.prototype.edgeCN;
    /** @type {?} */
    EdgeCentralConfig.prototype.ecvcHost;
    /** @type {?} */
    EdgeCentralConfig.prototype.inSecure;
}
/**
 * @record
 */
function EdgeCentralClientConfig() { }
if (false) {
    /** @type {?} */
    EdgeCentralClientConfig.prototype.apiUrl;
    /** @type {?} */
    EdgeCentralClientConfig.prototype.subscriptionUrl;
    /** @type {?} */
    EdgeCentralClientConfig.prototype.token;
}
class EdgeCentralClient {
    /* eslint-disable no-useless-constructor */
    /**
     * @param {?} config
     */
    constructor(config) {
        this.config = config;
        this.httpProtocol = 'https';
        this.wsProtocol = 'wss';
        this.store = createInstance({
            name: 'ec-client',
            driver: LOCALSTORAGE
        });
        if (config.inSecure) {
            // set protocol to insecure protocol
            this.httpProtocol = 'http';
            this.wsProtocol = 'ws';
        }
    }
    /**
     * @return {?}
     */
    isAuthenticated() {
        return __awaiter(this, void 0, void 0, function* () {
            // TODO: use more shopisticated detection mechanism
            yield delay(500);
            /* eslint-disable no-console */
            console.log('using chrome extensions?', this.isUsingChromeExtension());
            if (this.isUsingChromeExtension()) {
                return true;
            }
            /** @type {?} */
            const config = yield this.getClientConfig();
            if (!config.token) {
                return false;
            }
            return true;
        });
    }
    /**
     * return true when chrome inject script from ec-lite extenstion are presents
     * @return {?}
     */
    isUsingChromeExtension() {
        if (window.eclite) {
            return true;
        }
        return false;
    }
    /**
     * @return {?}
     */
    loadApolloClient() {
        return __awaiter(this, void 0, void 0, function* () {
            this.client = yield this.createApolloClient();
            console.log('client loaded!', this.client);
        });
    }
    /**
     * @return {?}
     */
    createApolloClient() {
        return __awaiter(this, void 0, void 0, function* () {
            // TODO: use more shopisticated detection mechanism
            yield delay(500);
            // get client configurations
            /** @type {?} */
            const config = yield this.getClientConfig();
            // HTTP connection to the API
            /** @type {?} */
            const httpLink = createHttpLink({
                uri: config.apiUrl,
                fetch: (/**
                 * @param {?} input
                 * @param {?} init
                 * @return {?}
                 */
                (input, init) => __awaiter(this, void 0, void 0, function* () {
                    /** @type {?} */
                    const conf = yield this.getClientConfig();
                    /** @type {?} */
                    const url = conf.apiUrl || input;
                    // set authorization header
                    if (init.headers && init.headers.set) {
                        init.headers.set('Authorization', `Bearer ${conf.token}`);
                    }
                    else {
                        init.headers = init.headers || ((/** @type {?} */ ({})));
                        init.headers.Authorization = `Bearer ${conf.token}`;
                    }
                    // using eclite offline first fetch
                    // when chrome extension are present
                    if (this.isUsingChromeExtension()) {
                        return window.eclite.fetch(url, init);
                    }
                    return fetch(url, init);
                }))
            });
            // subscriptions link
            /** @type {?} */
            const wsClient = new SubscriptionClient(config.subscriptionUrl, {
                reconnect: true
            }, 
            // using eclite offline first websocket
            // when chrome extension are present
            this.isUsingChromeExtension() ? window.eclite.WebSocket : WebSocket);
            wsClient.use([
                {
                    applyMiddleware: (/**
                     * @param {?} operationOptions
                     * @param {?} next
                     * @return {?}
                     */
                    (operationOptions, next) => __awaiter(this, void 0, void 0, function* () {
                        /** @type {?} */
                        const conf = yield this.getClientConfig();
                        operationOptions.token = conf.token;
                        next();
                    }))
                }
            ]);
            /** @type {?} */
            const wsLink = new WebSocketLink(wsClient);
            // Cache implementation
            /** @type {?} */
            const cache = new InMemoryCache();
            // using the ability to split links, you can send data to each link
            // depending on what kind of operation is being sent
            /** @type {?} */
            const link = split((
            // split based on operation type
            /**
             * @param {?} __0
             * @return {?}
             */
            ({ query }) => {
                /** @type {?} */
                const definition = getMainDefinition(query);
                return (definition.kind === 'OperationDefinition' &&
                    definition.operation === 'subscription');
            }), wsLink, httpLink);
            // Create apollo client
            /** @type {?} */
            const client = new ApolloClient({
                link,
                cache
            });
            return client;
        });
    }
    /**
     * @param {?} config
     * @return {?}
     */
    setClientConfig(config) {
        return this.store.setItem('config', config);
    }
    /**
     * @return {?}
     */
    getClientConfig() {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const config = yield this.store.getItem('config');
            // using default config when config not present
            if (!config) {
                return {
                    apiUrl: `https://${this.config.ecvcHost}/graphql`,
                    subscriptionUrl: `wss://${this.config.ecvcHost}/subscriptions`,
                    token: null
                };
            }
            return config;
        });
    }
    /**
     * logout and clear client contexts
     * @return {?}
     */
    logout() {
        return this.store.removeItem('config');
    }
    /**
     * @param {?} providerId
     * @param {?} password
     * @return {?}
     */
    loginByProvider(providerId, password) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const res = yield fetch(`${this.httpProtocol}://${this.config.cloudCentralHost}/login-provider`, {
                method: 'POST',
                mode: 'cors',
                headers: {
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify({
                    provider_id: providerId,
                    password
                })
            });
            if (!res.ok) {
                /** @type {?} */
                let errMessage = 'Gagal melakukan login';
                switch (res.status) {
                    case 404:
                    case 403:
                    case 401:
                        errMessage = 'Kode Faskes & Password tidak cocok';
                        break;
                    case 500:
                        errMessage = 'Maaf, Server kami mengalami gangguan';
                        break;
                }
                throw new Error(errMessage);
            }
            // get token
            const { token } = yield res.json();
            // apply configurations
            yield this.setClientConfig({
                apiUrl: `${this.httpProtocol}://${this.config.ecvcHost}/graphql`,
                subscriptionUrl: `${this.wsProtocol}://${this.config.ecvcHost}/subscriptions`,
                token
            });
        });
    }
    /**
     * @param {?} deviceId
     * @return {?}
     */
    loginByDevice(deviceId) {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const apiUrl = `${this.httpProtocol}://${deviceId}.${this.config.edgeCN}/graphql`;
            /** @type {?} */
            const subscriptionUrl = `${this.wsProtocol}://${deviceId}.${this.config.edgeCN}/subscriptions`;
            // test connection to edge central
            try {
                yield fetch(apiUrl, {
                    method: 'POST',
                    mode: 'cors',
                    headers: {
                        'Content-Type': 'application/json'
                    },
                    body: JSON.stringify({
                        query: `{
            __schema {
              types {
                name
              }
            }
          }`,
                        operationName: null,
                        variables: {}
                    })
                });
            }
            catch (err) {
                throw new Error('Gagal mendapatkan koneksi, pastikan device anda dalam satu jaringan');
            }
            // apply configurations
            yield this.setClientConfig({
                apiUrl,
                subscriptionUrl,
                token: '--ec-on-device-token--'
            });
        });
    }
}
if (false) {
    /**
     * @type {?}
     * @private
     */
    EdgeCentralClient.prototype.store;
    /** @type {?} */
    EdgeCentralClient.prototype.client;
    /**
     * @type {?}
     * @private
     */
    EdgeCentralClient.prototype.httpProtocol;
    /**
     * @type {?}
     * @private
     */
    EdgeCentralClient.prototype.wsProtocol;
    /**
     * @type {?}
     * @private
     */
    EdgeCentralClient.prototype.config;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/services/ec.service-provider.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * wrapped edge central client so it injectable
 * @param {?} config
 * @return {?}
 */
function provideEdgeCentralService(config) {
    return new EdgeCentralClient(config);
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/services/auth-guard.service.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class EdgeCentralGuardService {
    /**
     * @param {?} ec
     * @param {?} router
     */
    constructor(ec, router) {
        this.ec = ec;
        this.router = router;
    }
    /**
     * Return true when user are authenticated
     * @return {?}
     */
    canActivate() {
        return __awaiter(this, void 0, void 0, function* () {
            if (yield this.ec.isAuthenticated()) {
                return true;
            }
            this.router.navigate(['/login']);
            return false;
        });
    }
}
EdgeCentralGuardService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
EdgeCentralGuardService.ctorParameters = () => [
    { type: EdgeCentralClient },
    { type: Router }
];
if (false) {
    /**
     * @type {?}
     * @private
     */
    EdgeCentralGuardService.prototype.ec;
    /**
     * @type {?}
     * @private
     */
    EdgeCentralGuardService.prototype.router;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/components/login.page.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class LoginPage {
    /**
     * @param {?} toast
     * @param {?} loading
     * @param {?} router
     * @param {?} ec
     */
    constructor(toast, loading, router, ec) {
        this.toast = toast;
        this.loading = loading;
        this.router = router;
        this.ec = ec;
        this.tab = 'device';
    }
    /**
     * @param {?} event
     * @return {?}
     */
    tabChanged(event) {
        this.tab = event.detail.value;
    }
    /**
     * @return {?}
     */
    loginDevice() {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const loading = yield this.loading.create({
                message: 'Menghubungkan...'
            });
            yield loading.present();
            try {
                yield this.ec.loginByDevice(this.deviceID);
                yield delay(1000);
                yield loading.dismiss();
                // redirect to root page
                this.router.navigate(['/']);
            }
            catch (err) {
                /** @type {?} */
                const toast = yield this.toast.create({
                    message: err.message || err.toString(),
                    color: 'danger',
                    duration: 5000
                });
                yield toast.present();
            }
            yield loading.dismiss();
        });
    }
    /**
     * @return {?}
     */
    loginProvider() {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const loading = yield this.loading.create({
                message: 'Menghubungkan...'
            });
            yield loading.present();
            try {
                yield this.ec.loginByProvider(this.providerID, this.password);
                yield delay(1000);
                yield loading.dismiss();
                // redirect to root page
                this.router.navigate(['/']);
            }
            catch (err) {
                /** @type {?} */
                const toast = yield this.toast.create({
                    message: err.message || err.toString(),
                    color: 'danger',
                    duration: 5000
                });
                yield toast.present();
            }
            yield loading.dismiss();
        });
    }
}
LoginPage.decorators = [
    { type: Component, args: [{
                selector: 'sr-login-page',
                template: "<ion-header translucent>\r\n  <ion-toolbar>\r\n    <ion-title>Login</ion-title>\r\n  </ion-toolbar>\r\n</ion-header>\r\n\r\n<ion-content>\r\n  <ion-grid>\r\n    <ion-row class=\"ion-align-items-center ion-justify-content-center\">\r\n      <ion-col size-md=\"8\" size-sm>\r\n        <ion-card>\r\n          <ion-card-header>\r\n            <ion-segment (ionChange)=\"tabChanged($event)\" [value]=\"tab\">\r\n              <ion-segment-button value=\"device\" id=\"login-device\">\r\n                <ion-label>Login dengan kode perangkat</ion-label>\r\n              </ion-segment-button>\r\n              <ion-segment-button value=\"provider\" id=\"login-provider\">\r\n                <ion-label>Login dengan kode faskes</ion-label>\r\n              </ion-segment-button>\r\n            </ion-segment>\r\n          </ion-card-header>\r\n\r\n          <ion-card-content>\r\n            <ng-container *ngIf=\"tab === 'device'\">\r\n              <form ngForm #loginDeviceForm=\"ngForm\">\r\n                <ion-list lines=\"full\" class=\"ion-no-margin ion-no-padding\">\r\n                  <ion-item>\r\n                    <ion-label position=\"floating\">\r\n                      kode perangkat\r\n                      <ion-text\r\n                        color=\"danger\"\r\n                        *ngIf=\"device_id.invalid && (device_id.dirty || device_id.touched)\"\r\n                      >\r\n                        <span *ngIf=\"device_id?.errors?.required\">\r\n                          wajib diisi\r\n                        </span>\r\n                      </ion-text>\r\n                    </ion-label>\r\n                    <ion-input\r\n                      name=\"device_id\"\r\n                      [(ngModel)]=\"deviceID\"\r\n                      type=\"text\"\r\n                      placeholder=\"misal prkt123\"\r\n                      required\r\n                      #device_id=\"ngModel\"\r\n                      id=\"device-id\"\r\n                    ></ion-input>\r\n                  </ion-item>\r\n                </ion-list>\r\n                <div class=\"ion-padding\">\r\n                  <ion-button\r\n                    shape=\"round\"\r\n                    expand=\"block\"\r\n                    color=\"primary\"\r\n                    (click)=\"loginDevice()\"\r\n                    [disabled]=\"!loginDeviceForm.form.valid\"\r\n                    id=\"submit-device\"\r\n                  >\r\n                    <ion-icon slot=\"start\" name=\"unlock\"></ion-icon>\r\n                    Login\r\n                  </ion-button>\r\n                </div>\r\n              </form>\r\n            </ng-container>\r\n\r\n            <ng-container *ngIf=\"tab === 'provider'\">\r\n              <form ngForm #loginProviderForm=\"ngForm\">\r\n                <ion-list lines=\"full\" class=\"ion-no-margin ion-no-padding\">\r\n                  <ion-item>\r\n                    <ion-label position=\"floating\">\r\n                      kode faskes\r\n                      <ion-text\r\n                        color=\"danger\"\r\n                        *ngIf=\"provider_id.invalid && (provider_id.dirty || provider_id.touched)\"\r\n                      >\r\n                        <span *ngIf=\"provider_id?.errors?.required\">\r\n                          wajib diisi\r\n                        </span>\r\n                      </ion-text>\r\n                    </ion-label>\r\n                    <ion-input\r\n                      name=\"provider_id\"\r\n                      [(ngModel)]=\"providerID\"\r\n                      type=\"text\"\r\n                      placeholder=\"misal klinik_sehat\"\r\n                      required\r\n                      #provider_id=\"ngModel\"\r\n                      id=\"provider-code\"\r\n                    ></ion-input>\r\n                  </ion-item>\r\n\r\n                  <ion-item>\r\n                    <ion-label position=\"floating\">\r\n                      password\r\n                      <ion-text\r\n                        color=\"danger\"\r\n                        *ngIf=\"passwords.invalid && (passwords.dirty || passwords.touched)\"\r\n                      >\r\n                        <span *ngIf=\"passwords?.errors?.required\">\r\n                          wajib diisi\r\n                        </span>\r\n                      </ion-text>\r\n                    </ion-label>\r\n                    <ion-input\r\n                      name=\"password\"\r\n                      [(ngModel)]=\"password\"\r\n                      type=\"password\"\r\n                      required\r\n                      #passwords=\"ngModel\"\r\n                      id=\"provider-password\"\r\n                    ></ion-input>\r\n                  </ion-item>\r\n                </ion-list>\r\n\r\n                <div class=\"ion-padding\">\r\n                  <ion-button\r\n                    shape=\"round\"\r\n                    expand=\"block\"\r\n                    color=\"primary\"\r\n                    (click)=\"loginProvider()\"\r\n                    [disabled]=\"!loginProviderForm.form.valid\"\r\n                    id=\"submit-provider\"\r\n                  >\r\n                    <ion-icon slot=\"start\" name=\"unlock\"></ion-icon>\r\n                    Login\r\n                  </ion-button>\r\n                </div>\r\n              </form>\r\n            </ng-container>\r\n          </ion-card-content>\r\n        </ion-card>\r\n      </ion-col>\r\n    </ion-row>\r\n  </ion-grid>\r\n</ion-content>\r\n"
            }] }
];
/** @nocollapse */
LoginPage.ctorParameters = () => [
    { type: ToastController },
    { type: LoadingController },
    { type: Router },
    { type: EdgeCentralClient }
];
if (false) {
    /** @type {?} */
    LoginPage.prototype.tab;
    /** @type {?} */
    LoginPage.prototype.deviceID;
    /** @type {?} */
    LoginPage.prototype.providerID;
    /** @type {?} */
    LoginPage.prototype.password;
    /**
     * @type {?}
     * @private
     */
    LoginPage.prototype.toast;
    /**
     * @type {?}
     * @private
     */
    LoginPage.prototype.loading;
    /**
     * @type {?}
     * @private
     */
    LoginPage.prototype.router;
    /**
     * @type {?}
     * @private
     */
    LoginPage.prototype.ec;
}

/**
 * @fileoverview added by tsickle
 * Generated from: lib/ec.module.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} ec
 * @return {?}
 */
function initEdgeCentralClient(ec) {
    /** @type {?} */
    const loader = (/**
     * @return {?}
     */
    () => ec.loadApolloClient());
    return loader;
}
const ɵ0 = provideEdgeCentralService;
/**
 *
 * @export
 */
class EdgeCentralModule {
    /**
     * Creates an instance of EdgeCentralModule.
     * throw error if EdgeCentralModule already created on parent
     * @param {?} parentModule
     */
    constructor(parentModule) {
        if (parentModule) {
            throw new Error('EdgeCentralModule is already loaded. Import it in the AppModule only');
        }
    }
    /**
     * Create new module instance with edge central config service
     *
     * @param {?} config
     * @return {?}
     */
    static forRoot(config) {
        return {
            ngModule: EdgeCentralModule,
            providers: [{ provide: EdgeCentralConfig, useValue: config }]
        };
    }
}
EdgeCentralModule.decorators = [
    { type: NgModule, args: [{
                imports: [IonicModule.forRoot({ mode: 'ios' })],
                providers: [
                    {
                        provide: EdgeCentralClient,
                        useFactory: ɵ0,
                        deps: [EdgeCentralConfig]
                    },
                    {
                        provide: APP_INITIALIZER,
                        useFactory: initEdgeCentralClient,
                        deps: [EdgeCentralClient],
                        multi: true
                    },
                    EdgeCentralGuardService
                ]
            },] }
];
/** @nocollapse */
EdgeCentralModule.ctorParameters = () => [
    { type: EdgeCentralModule, decorators: [{ type: Optional }, { type: SkipSelf }] }
];

/**
 * @fileoverview added by tsickle
 * Generated from: lib/ec-login.router.module.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const routes = [{ path: '', component: LoginPage }];
/**
 * Login Router modules
 * @export
 */
class EdgeCentralLoginRoutingModule {
}
EdgeCentralLoginRoutingModule.decorators = [
    { type: NgModule, args: [{
                imports: [RouterModule.forChild(routes)],
                exports: [],
                declarations: []
            },] }
];
/** @type {?} */
const routedComponents = [LoginPage];

/**
 * @fileoverview added by tsickle
 * Generated from: lib/ec-login.module.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Module to provide context user interface before accessing edge central features
 *
 * @export
 */
class EdgeCentralLoginModule {
}
EdgeCentralLoginModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    IonicModule,
                    CommonModule,
                    EdgeCentralLoginRoutingModule,
                    FormsModule,
                    ReactiveFormsModule
                ],
                declarations: [...routedComponents]
            },] }
];

/**
 * @fileoverview added by tsickle
 * Generated from: public-api.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * Generated from: sirusdev-ec-client-angular.ts
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { EdgeCentralClient, EdgeCentralConfig, EdgeCentralGuardService, EdgeCentralLoginModule, EdgeCentralModule, LoginPage, initEdgeCentralClient, provideEdgeCentralService, EdgeCentralLoginRoutingModule as ɵa, routedComponents as ɵb };
//# sourceMappingURL=sirusdev-ec-client-angular.js.map
